from classes import weapon, armor
import random
from code import util, terminal
from battle import buff


class Enemy():
    def __init__(self, name, life, magic, weapon, armor, can_magic=False, can_heal=False, silver=0, gold=0, diamonds=0, buff=False):
        """Initate the Enemy Class"""
        self.name = name
        self.life = life
        self.max_life = life
        self.magic = magic
        self.max_magic = magic
        self.weapon = weapon
        self.armor = armor
        self.can_magic = can_magic
        self.buff = buff
        self.silver = silver
        self.gold = gold
        self.diamonds = diamonds

        self.can_heal = can_heal


        self.dead = False


    def get_battle_stats(self):
        """Get the Stats important to battle"""

        stats = f"LIFE: {str(self.life)}/{str(self.max_life)}"

        if self.can_magic:
            stats += f" | MAGIC: {str(self.magic)}/{str(self.max_magic)}"

        return stats


    def desperate_attack(self, player):
        """Attack the Player in a last ditch effort -- No Buffs"""

        green = "\033[32m"
        red = "\033[31m"
        reset = "\033[0m"

        terminal.slow_print(f"The {self.name.capitalize()} Charges!")

        dmg = (self.weapon.damage+self.weapon.sharpness)*self.weapon.weight

        terminal.slow_print(f"{red}+{str(dmg)}{reset}")

        true_dmg = (self.weapon.damage+util.floor(self.weapon.sharpness-player.armor.armor))*self.weapon.weight

        armor_block = dmg-true_dmg

        terminal.slow_print(f"{green}   -{str(armor_block)} -- armor{reset}")

        if dmg > 0:
            terminal.slow_print(f"{red}{true_dmg} Damage{reset}")
            player.life -= true_dmg
        else:
            terminal.slow_print(f"{green}0{reset}")


    def heal(self, player, player_turn, buffs, enemies):
        """Attempt to heal self using spell `sana ego` or `sana vulnera`"""
        if self.magic >= 5:
            terminal.slow_print(f"{self.name} heals")
            self.magic -= 5
            self.life += 15
            terminal.slow_print("+15")
        elif self.magic == 4:
            terminal.slow_print(f"{self.name} heals")
            self.magic -= 4
            self.life += 10
            terminal.slow_print("+10")
        else:
            self.choose_attack(player, player_turn, buffs, enemies)


    def choose_attack(self, player, player_turn, buffs, enemies):
        """Pick an attack based on the player and battle position/fellow enemies"""

        green = "\033[32m"
        red = "\033[31m"
        reset = "\033[0m"

        # Defualt to attack the player
        # check if buddies
        # ENEMIES CAN NO LONGER SUMMON

        # if len(enemies) == 1:
        #     # if no buddies and can sommon, summon buddies
        #     if self.can_summon and not summoned:
        #         self.summon(enemies)
        #         summoned = True
        #         return
        #     # else return to process

        # calc damage

        raw_dmg = (self.weapon.damage+self.weapon.sharpness)*self.weapon.weight
        true_dmg = (self.weapon.damage+util.floor(self.weapon.sharpness-player.armor.armor))*self.weapon.weight

        # if player is one hit away from dead. Finish them
        # NOTE: This is sort of OP

        if player.life <= true_dmg:
            terminal.slow_print(f"{self.name} attacks")
            terminal.slow_print(f"+{str(raw_dmg)}")
            terminal.slow_print(f"   -{str(raw_dmg-true_dmg)} -- armor")

            for buff in buffs:
                true_dmg += buff.enemy_dmg
                if buff.enemy_dmg > 0:
                    terminal.slow_print(f"{red}   +{str(buff.enemy_dmg)} --{buff.reason}{reset}")
                else:
                    terminal.slow_print(f"{green}   -{str(buff.enemy_dmg)} --{buff.reason}{reset}")

            if true_dmg < 0:
                true_dmg = 0

            terminal.slow_print(f" {red}{str(true_dmg)} Damage{reset}")

            player.life -= true_dmg

            return

        # TODO: if magic and magic attack and magic attack is enough to finish them damage than use magic
            
        # if more then two buddies and buff available, use buff

        if len(enemies) >= 3 and self.buff:
            terminal.slow_print(f"{self.name} used {self.buff.reason}")
            terminal.slow_print(self.buff.des)
            buffs.append(self.buff)
            self.buff = False
            return

        # TODO: if magic more damage then weapon and enough magic to throw, throw spell

        # TODO: if inventory item more damage then weapon use inventory item

        # TODO: if stamina use weapon

        terminal.slow_print(f"{self.name} attacks")
        terminal.slow_print(f"+{str(raw_dmg)}")
        green = "\033[32m"
        terminal.slow_print(f"{green}   -{str(raw_dmg-true_dmg)} -- armor{reset}")

        for buff in buffs:
            true_dmg += buff.enemy_dmg
            if buff.enemy_dmg > 0:
                terminal.slow_print(f"{red}   +{str(buff.enemy_dmg)} --{buff.reason}{reset}")
            else:
                terminal.slow_print(f"{green}   -{str(buff.enemy_dmg)} --{buff.reason}{reset}")

        if true_dmg < 0:
            true_dmg = 0

        terminal.slow_print(f" {red}{str(true_dmg)} Damage{reset}")

        player.life -= true_dmg

        return

        # TODO: if no stamina look for inventory item

            # TODO: if potion of energy found use over weapon

        # TODO: if no inventory item rest

        

        
        
        
    


from people import chat
from classes import weapon
import random
from code import util, terminal
from people import chat


class Laurence():
    def __init__(self):
        self.symbol = 'L'

        self.voice = chat.Chat("Laurence ", "\033[34m")

        

    def speak(self, player):
        terminal.clear()
        if player.laurence_talked_to:
            self.voice.say("'ello there! Good to see you again, what wish ye to speak about?")
            player.voice.say("Any news on that spell?")
            self.voice.say("I'm afraid not, without Ferric I have hit a block in my knowledge of the magic arts")

        else:
            self.introduce(player)


    def introduce(self, player):
        terminal.clear()
        player.laurence_talked_to = True

        self.voice.say("Hello? Is someone there?")
        player.voice.say("It's me... a fellow survivor")
        self.voice.say("You aren't one of the crazy ones are you?")
        player.voice.say("I do not think that I am")
        self.voice.say("That is good then, I am Laurence, what is your name?")
        player.voice.say("I... I can't remember")
        self.voice.say("That's a shame.")
        player.voice.say("Do you know who I am?")
        self.voice.say("No no... Not that I can remember... I forget many things though")
        player.voice.say("Do you know much about this place?")
        self.voice.say("Oh yes... I would think so")
        player.voice.say("How long have you been here?")
        self.voice.say("Three thousand nine hundred and forty one days, fifteen hours and... thirty two minutes")
        player.voice.say("How do you know?")
        self.voice.say("I calculate the time based on the geometric angle the portals form at the shifting of the dungeon")
        player.voice.say("You seem very smart")
        self.voice.say("I used to think so... Not now, I've forgotten the one thing that could of helped me escape here")
        player.voice.say("A way out?")
        self.voice.say("Not a path out, but a spell... It's a long story")
        player.voice.say("Tell me!")
        self.voice.say("Very well...")
        terminal.clear()
        self.voice.say("I have studied the dungeon and deduced that it is always shifing, any level that is not occupied by a life form has it's shape shifted.")
        self.voice.say("Each time we enter a portal the other end connects us to a random room in the portal")
        player.voice.say("Then where is the end?")
        self.voice.say("There is no end, by my calculations there are three hundred and forty million six hundred and- well you get the point. The possible configurations of the dungeon make it mathmatically impossible to find the heart of magic")
        player.voice.say("The heart of magic?")
        self.voice.say("Yes... isn't that we are all here for?")
        player.voice.say("Do you know where it is?")
        self.voice.say("No... entering each portal you have a very small chance of entering the room where the heart is stored, by my calculations it is mathmatically impossible to get there")
        player.voice.say("You spoke of an exit?")
        self.voice.say("Ah yes... the spell.")
        player.voice.say("A spell?")
        self.voice.say("Yes, nearly a thousand years ago a man by the name of Ferric came here. He was a wise scholar and powerful wizard. Together we worked to fashion a spell that would open us a portal back to the mortal realm")
        player.voice.say("Did it work?")
        self.voice.say("I'm not sure, Ferric was killed by a ghoul when the spell was nearly complete. I finished it after his death, but not being a wizard myself I could not throw it")
        player.voice.say("I'm a wizard, what is the spell?")
        self.voice.say("I'm sorry... Over the years I've forgotten the entire spell, I was such a fool and never wrote it down, thinking I could remember it")
        player.voice.say("Can't you rebuild it?")
        self.voice.say("I'm a mathmatician not a wizard. Ferric built most of the spell I added a little, mostly the calculations needed to set a correct location and such... without his knowledge I can't even come close to a working spell")
        player.voice.say("If only you could remember it, or remake it")
        self.voice.say("You are a wizard, perhaps you can, I still have Ferric's journal, it does not contain the spell but it does contain his thoughts. Most of them a spells which I do not understand but it could aide you.")
        player.inventory.append("Ferric's Journal")
        player.voice.say("Thankyou friend, I'm sure his journal will be a great help to me")
        self.voice.say("I hope it is. Sadly most of the pages have been destroyed or fallen out")
        self.voice.say("Ferric was a great man if he had survived I'm sure I would not be here")
        player.voice.say("What can you tell me about the heart of magic?")
        self.voice.say("The heart? It is the most powerful peice of magic ever, it will allow anyone who takes it a single action. According to legend they can bring back loved ones, change the travel of time, destroy or create, those are only legends however, and nobody knows it's true power")
        player.voice.say("Is everyone here seeking it?")
        self.voice.say("Nearly all of us came becuase of it. Death is no fool however, he built this dungeon beneath his castle to hide the heart as it is the only thing that can destroy him.")
        player.voice.say("Did you come here after it?")
        self.voice.say("Yes I did... although I wish I had not")
        player.voice.say("If you found it what would you use it for?")
        self.voice.say("I would hope i would leave it be. Things of such power were not meant for anyone to have. My poor choices have left me in this despicable place, I deserve this fate. Using that thing, i feel would only lead me into more horrible places, too much power will currupt anyone, no matter how strong they are")
        player.voice.say("You don't want to leave here?")
        self.voice.say("I've been here for thousands of years. I will do nearly anything to escape, but I will not let the heart be part of it. It would only bring my to ruin with it's power. I know there is an exit from this place. I have entered enough portals that mathmatically it is possible in the next few hundred years I will find that exit.")
        player.voice.say("I wish you well then Laurence")
        self.voice.say('You as well my friend')
            




    def choice(self, prompt, valid):
        """Todo: Update to Navigate Function"""
        while True:
            print(prompt)
            i = input()
    
            if i in valid:
                return i




from battle import buff, die
from code import terminal, util
from classes import weapon, armor
from people import chat



class Warren():
	def __init__(self):
		"""Initate Warren"""
		self.name = "Warren"
		self.life = 500 # 500
		self.max_life = 500
		self.weapon = weapon.Weapon("Kakavian Longsword", 10, 10, 10)
		self.armor = armor.Armor("Kakavian Battle Armor", 10)
		self.voice = chat.Chat("Warren: ", "\033[34m")

	def calc_hit(self, player):
		"""Calculate Damage"""
		terminal.clear()

		green = "\033[32m"
		red = "\033[31m"
		reset = "\033[0m"

		raw_dmg = (self.weapon.damage+self.weapon.sharpness)*self.weapon.weight

		terminal.slow_print(f"{green}+{str(raw_dmg)}{reset}")

		dmg = (self.weapon.damage+util.floor(self.weapon.sharpness - player.armor.armor))*self.weapon.weight

		armor_block = raw_dmg-dmg

		terminal.slow_print(f"{red}   -{str(armor_block)} -- armor{reset}")

		if dmg > 0:
			terminal.slow_print(f" {green}{str(dmg)} Damage{reset}")
		else:
			dmg = 0
			terminal.slow_print(f" {red}{str(dmg)} Damage{reset}")

		return dmg

	def get_stats(self):
		"""Return life... kinda lame"""
		return f"WARREN -- LIFE: {str(self.life)}/{str(self.max_life)}"


	def attack(self, player):
		"""Attack the Player"""

		if random.range(1, 5) == 1:
			dmg = self.calc_dmg(player)

			player.life -= dmg

		else:
			dmg = int(.9*player.life)
			terminal.slow_print(f"Super Strike: {str(dmg)} damage")

			player.life -= dmg

	def win(self, player):
		"""Warren Wins"""
		terminal.clear()

		self.voice.say("You are defeated Mikkel... Your even weaker then I remember.")
		player.voice.say("And you... are stronger then I remember...")
		self.voice.say("I warned you what would happen here Mikkel. I told you I would kill you")
		player.voice.say("You are a traitor...")
		self.voice.say("Yes... I'm a traitor, but I don't care! It's for my family Mikkel. I don't expect you to understand...")
		player.voice.say("I'm sorry about what happened to them, but you don't have to do this")
		self.voice.say("Heh... Do you know why I attacked you in the first place? Why I started fighting against you?")
		player.voice.say("No.")
		self.voice.say("You promised me you were going to take those letters to my son. You never told me he was dead!")
		player.voice.say("I'm sorry... I thought it would be better-")
		self.voice.say("If I didn't know? Ha, well how did that turn out of you")
		player.voice.say("I'm sorry Warren, I failed you")
		self.voice.say("Not just me. You failed to protect Coyote, William, Trekker, Ferric, you failed all of us. Your loved ones in the mortal realm, everyone, you failed them all")
		player.voice.say("I-")
		self.voice.say("You and your stupid gate. Guarding that gate cost more lives then would have been lost if we had just let people sort themselves out! After all, what have they done to deserve our protection? I say let them protect themselves! I have no reason to care for them")
		player.voice.say("Then you are a selfish boy, men protect the weak")
		self.voice.say("Even if you had one this battle my friend, it woudln't have mattered. Death and life are going to exist in all ages. No matter what you do to lock this place poeple will get here eventually, you can only delay the inevitable ruin. My wife and son, family, loved ones, those are things that will not always exist, they only last a short period of time, I just want to bring them back them back for just a few more days. To right my wrongs")
		player.voice.say("Even if you have to do more wrong to do it?")
		self.voice.say("You wouldn't understand Mikkel, and sadly you won't ever have the time to figure out")
		player.voice.say("Your wrong about that Warren")
		self.voice.say("Your a fool! Any last words?")
		words = input("You: ")
		self.voice.say("Goodbye Mikkel.")
		die.die("Killed by Warren Lightoot.")

	def loose(self, player):
		"""Warren Looses."""

		terminal.clear()

		player.voice.say("Give up Warren, you are defeated!")
		self.voice.say("You'll have to kill me Mikkel! I will never surrender!")
		player.voice.say("It must be done Warren, I can use that thing to destroy the gate forever, to protect our home forever, we must protect the mortal realm, it was our oath!")
		self.voice.say("Destroying the gate will trap us here forever! I have been here far too long, I cannnot live another day!")
		player.voice.say("I too fear remaining in this place, but it must be done!")
		self.voice.say("Kill me then.")
		player.voice.say("I don't want to kill you...")
		self.voice.say("DO IT! You Coward! Kill me! I will not live another day down here, do it or I'll kill myself!")
		player.voice.say("I miss my family as well Warren, you were not the only one who lost loved ones.")
		self.voice.say("Not the same way I did. If you were in my place you would use that stone to bring them back, you would do what I have done!")
		player.voice.say("I want to Warren, I want to bring them back, but I know I cannot. I must destroy the gate, it is what they would want us to do")
		self.voice.say("I just want to see them once more Mikkel! To speak to my son, hug my wife, just for one more day...")
		player.voice.say("I'm sorry Warren, it must be done. We can find another way out, together")
		self.voice.say("There is no way out except the gate, destroying it will leave us here forever!")
		player.voice.say("There is always another way out")
		self.voice.say("If there is a way out, there is a way in! Listen to me Mikkel. Destroying that gate, using a spell, those things do nothing! They only delay the inevitable. Some things were just meant to be. Death, life, this place and our home will always exist. The things that do not exist forever are our loved ones. My Wife. My Son. Ferric's family, Coyote's brother, Williams sister, Sharie. All of them are the things that pass away. Bring them back, even if just for a day. It's a better use then delaying the destruction of our mortal realm a few more short years. You can't change the future Mikkel, whatever you do death will just find another way to escape. What you can change is the past! You can bring them back, all of them!")
		player.voice.say(". . .")

		print("[1] Warren is right. Bring them all back")
		print("[2] Destroy the gate. The few must sacrifice for the good of the many")

		choice = util.get_input("-> ", valid=['1', '2'])

		print("The End...")
		print("I hope you had fun playing")

		exit()





warren = Warren()


def fight(player):
	"""Fight between Player and Warren"""

	player_turn = True # player always start, might switch to player_turn = random.randint(0, 1)

	valid_nums = [str(i) for i in range(0, 9)]

	buffs = []

	last_hit = None

	green = "\033[32m"
	red = "\033[31m"
	reset = "\033[0m"


	while player.life > 0 and warren.life > 0:
		while player_turn:
			terminal.clear()
			print(player.get_stats())
			print("\n\n---------- BUFFS -----------\n")
			for b in buffs:
				if not b.enemy:
					print(f"{red}{b.reason} -> {b.des}{reset}")
				else:
					print(f"{green}{b.reason} -> {b.des}{reset}")
			if not buffs:
				print("None")
			print("\n------------------------------")
			print("\n\n---------- WARREN -----------\n")
			print(warren.get_stats())
			print("\n-----------------------------")

			commands = ['?', 'attack', 'loadout', 'drink', 'magicae', 'run']
			
			full = input("-> ")

			c = full.split()

			command = c[0]

			if command in commands:
				# use try to invalid commands
				if command == '?':
					print("Figure it out loser!")
				
				elif command == 'attack':
					player.attack(warren, buffs, [warren], noloot=True)
					player_turn = False
				
				elif command == 'loadout':
					player.shuffle_loadout()
				
				elif command == 'run':
					warren.voice.say("You fool. You can't run away from this!")
				
				elif command == 'drink':
					potion = " ".join(c[1:]).lower()

					if potion in player.inventory and potion in randomness.valid_potions:
						player.inventory.remove(potion)
						
						if potion == 'potion of energy':
							b = buff.Buff(50, 0, 1, "Potentia Navitas", "+50 on next attack")
							print("Buff Activated: +50 on next attack")
							buffs.append(b)
							player_turn = False
						elif potion == 'healing potion':
							print("You drank the healing potion...")
							player.life = player.max_life
							player_turn = True
					else:
						print("You don't have that potion!")
				
				elif command == 'magicae':
					spell = " ".join(c[1:])
					if spell in randomness.valid_spells:
						if spell == 'sana ego':
							if player.magic >= 4:
								player.magic -= 4
								player.life += 10
								if player.life > player.max_life:
									player.life = player.max_life
							else:
								print("You do not have enough power to throw that spell")

						elif spell == 'sana vulnera':
							if player.magic >= 5:
								player.magic -= 5
								player.life += 15
								if player.life > player.max_life:
									player.life = player.max_life
							else:
								print("You do not have enough power to throw that spell")
						
						elif spell == 'sana omnis vulnera':
							if player.magic >= 10:
								player.magic -= 10
								player.life += 40
								if player.life > player.max_life:
									player.life = player.max_life
							else:
								print("You do not have enough power to throw that spell")
						
						elif spell == 'confirma impetum':
							if player.magic >= 10:
								player.magic -= 10
								b = buff.Buff(40, 0, 1, "Confirma Impetum", "+40 Damage on Next Attack")
								buffs.append(b)
								print("Buff Activated: +40 Damage on Next Attack")
							else:
								print("You do not have enough power to throw that spell")
						
						elif spell == 'confirma defensio':
							if player.magic >= 10:
								player.magic -= 10
								b = buff.Buff(0, -40, 1, "Confirma Defensio", "-40 Damage On Next Enemy Attack")
								buffs.append(b)
								print("Buff Activated: -40 Damage on Next Enemy Attack")
							else:
								print("You do not have enough power to throw that spell")
						
						elif spell == 'noxa hostilis':
							if player.magic >= 25:
								player.magic -= 25
								print("All Enemies Recieve 50 Damage")
								for enemy in enemies:
									enemy.life -= 50
									if enemy.life <= 0:
										enemies.remove(enemy)
								player_turn = False
							else:
								print("You do not have enough power to throw that spell")
					else:
						print("Your strange magic words are confusing...")
						print(spell + " is not a spell")
			util.cont()

		if warren.life <= 0:
			warren.loose(player)
		

		while not player_turn:
			player_turn = True
			warren.attack(player, buffs)

		for b in buffs:
			b.lasts -= 1
			if b.lasts <= 0:
				buffs.remove(b)

		if player.life <= 0:
			warren.win(player)
		

	print("Game Over")



from generators import dungeon
from code import game, map, terminal, util
from classes import player
from save_game import menu, save
import time, random, cursor

from battle import bossfight

cursor.hide()

util.clear_log()


##################
#  BUILD PEOPLE  #
##################

from people import maximus, grarlan, rohan, laurence, trekker, warren

maximus = maximus.Maximus()
grarlan = grarlan.Grarlan()
rohan = rohan.Rohan()
laurence = laurence.Laurence()
trekker = trekker.Trekker()
warren = warren.Warren()


characters = [maximus, grarlan, rohan, laurence, trekker, warren]






def build_dungeon(p):
    """Generates a Random Dungeon"""
    print("Loading Dungeon....")


    util.log("Initating Dungeon")

    gen = dungeon.Generator(
        width=100,
        height=100,
        max_rooms=15,
        min_room_xy=5,
        max_room_xy=15,
        rooms_overlap=False,
        random_connections=1,
        random_spurs=1
    )

    util.log("Generating Level")
    gen.gen_level()

    util.log("Generating Tiles")
    d = gen.gen_tiles_level()

    util.log("Adding Loot")
    map.add_loot(
        d,
        min_loot = 4,
        max_loot = 15,
    )

    util.log("Adding Exit Portal")
    map.add_exit(
        d
    )


    util.log("Adding Characters")
    
    if random.randint(1, 10) == 1:
        util.log("Placing M")
        map.add_character(d, maximus)

    if random.randint(1, 10) == 1:
        util.log("Placing G")
        map.add_character(d, grarlan)

    if random.randint(1, 10) == 1:
        util.log('Placing R')
        map.add_character(d, rohan)

    if random.randint(1, 10) == 1:
        util.log("Placing L")
        map.add_character(d, laurence)

    if random.randint(1, 10) == 1 and not p.trekker_alive:
        map.add_character(d, warren)

    if random.randint(1, 3) == 1 and p.trekker_alive:
        util.log("Placing T")
        map.add_character(d, trekker)

    util.log("Adding Enemies")

    for i in range(random.randint(3, 8)):
        map.add_enemy(d)
    
    util.log("Adding Shards")

    if random.randint(1, 4) == 1:
        map.add_shard(d)


    util.log("Dungeon Created")

    terminal.clear()
    
    return d




###############
#  INIT GAME  #
###############

save.initiate()

game_data = menu.menu()


if not game_data:
    player = player.Player()
    d = build_dungeon(player)

    g = game.Game(
        dungeon = d,
        player = player,
        characters = characters,
    )

elif game_data:
    g = game.Game(
        dungeon = game_data[1],
        player = game_data[0],
        characters = characters,
        posx = game_data[2],
        posy = game_data[3]
    )

else:
    print("Error! Game Both Does not exist and Exists")
    exit()




####################
#  MAIN GAME LOOP  #
####################


while True:
    go_next = g.play()
    if not go_next:
        print("Hope you had a fun time playing!")
        exit()
    else:
        g.player.rooms_played += 1
        if g.player.warren_talked_to:
            if g.player.rooms_played >= 20:
                # start checking for win room
                if random.randint(1, 10) == 1:
                    print("You won!") # TODO: Replace with Win Room/Boss Fight
                    exit()
        print("...")
        time.sleep(1) # just for effect
        terminal.clear()
        d = build_dungeon(g.player)
        g = game.Game(
            dungeon=d,
            player=g.player,
            characters=characters,
        )
        


import random

from people import maximus


screen_size = 10


def build_display_map(dungeon, x, y):
    """Cuts the full map into a size based on the users location"""
    # TODO: Fix this crap
    display_map = []
    for i in range(-screen_size, screen_size):
        try:
            display_map.append(dungeon[y+i])
        except:
            display_map.append(["."]*screen_size)
    final = []
    for row in display_map:
        new_row = []
        for i in range(-screen_size+x, screen_size+1+x):
            try:
                new_row.append(row[i])
            except:
                new_row.append(".")
                
        final.append(new_row)
    return final
    
    

def display(dungeon, x, y, stats=[]):
    print("\033[H",end="")
    dungeon[y][x] = 'p'
    _map = build_display_map(dungeon, x, y)
    print("---"*21+"--")
    for stat in stats:
        count = int((60-len(stat))/2)
        _str = "|  " + (" "*count) + stat + (" "*count)
        if len(_str) == 62:
            _str += "  |"
        else:
            _str += " |"

        print(_str)

        
    print("---"*21+"--")
    for row in _map:
        r = "|"
        for cell in row:
            if cell == 't':
                r += " []"
            elif cell == '*':
                r += "\033[34m * \033[0m"
            else:
                r += " "+cell+" "
        print(r+"|")
    print("---"*21+"--")


class action():
    def __init__(self, dungeon, x, y):
        self.dungeon = dungeon
        self.x = x
        self.y = y
        self.valid = False
        self.treasure = False
        self.portal = False
        self.character = False
        self.enemy = False
        self.shard = False


        self.check_position()

    def check_position(self):
        """Check the position of the character"""
        item = self.dungeon[self.y][self.x]

        if item in ['#', '@']:
            self.valid = False
        else:
            self.valid = True

        if item == 't':
            self.treasure = True

        if item == '@':
            self.portal = True

        if item in ['M', 'G', 'L', 'R', 'T', 'W']:
            self.valid = False
            self.character = item

        if item == 'e':
            self.enemy = True
            self.valid = False
        
        if item == '*':
            self.shard = True
        

def getSpawn(dungeon):
    valid = False
    count = 0
    spawny, spawnx = 0,0
    while not valid:
        try:
            if ' ' in dungeon[count]:
                spawny = count
                spawnx = random.randint(0, len(dungeon))
                if dungeon[spawny][spawnx] == ' ':
                    valid = True
            else:
                count += 1
        except:
            pass
    return spawny, spawnx


def check_pos(y, x, dungeon):
    """Checks if the position is valid for Character Placement"""

    # Checks if the character is in a legal spot
    # to keep them from blocking the way for the player
    # checks if the spot above, below, to the right and left
    # of the spawn being tested are clear. 
    # While this will not completely keep the character
    # from blocking the players movement. it should
    # make it a lot less likely to accur. Hopefull...

    pos = dungeon[y][x]
    if pos != ' ':
        return False
    else:
        spots = []

        #check position above
        spots.append(dungeon[y+1][x])

        # check position below
        spots.append(dungeon[y-1][x])

        # check position right
        spots.append(dungeon[y][x+1])

        # check position left
        spots.append(dungeon[y][x-1])

        for position in spots:
            if position != ' ':
                return False
    return True

def add_character(dungeon, character):
    """Add a character in random location"""
    # Note. Check Character to surroundings to make sure
    # they are not blocking the path of the character
    attempts = 0
    while 1:
        y = random.randint(0, 98)
        if ' ' in dungeon[y]:
            while 2: # never seen this before have you?
                attempts += 1
                x = random.randint(0, 98)
                if check_pos(y, x, dungeon):
                    dungeon[y][x] = character.symbol
                    return
                if attempts == 1000:
                    # Find First Open (And Valid) Spot
                    while True:
                        print("Switching to Linear Placement")
                        y = 0
                        if ' ' in dungeon[y]:
                            while True:
                                x = 0
                                if check_pos(y, x, dungeon):
                                    dungeon[y][x] = character.symbol
                                    return
                                else:
                                    x += 1
                        else:
                            y += 1

                        if y <= 98:
                            util.log("Placing Character " + str(character.symbol) + " failed.")

def add_enemy(dungeon):
    """Add an enemy in a random location"""
    valid = False
    while True:
        y = random.randint(0, 98)
        if ' ' in dungeon[y]:
            while True:
                if ' ' in dungeon[y]:
                    x = random.randint(0, 98)
                    if dungeon[y][x] == ' ':
                        dungeon[y][x] = 'e'
                        return


def add_shard(dungeon):
    """Adds a shard of the heart of Magic"""
    while True:
        y = random.randint(0, 98)
        if ' ' in dungeon[y]:
            while True:
                x = random.randint(0, 98)
                if dungeon[y][x] == ' ':
                    dungeon[y][x] = '*'
                    return


def add_loot(dungeon, min_loot, max_loot):
    loot_count = random.randint(min_loot, max_loot)

    for i in range(loot_count):
        while True:
            y = random.randint(0, 98)
            if ' ' in dungeon[y]:
                while True:
                    x = random.randint(0, 98)
                    if dungeon[y][x] == ' ':
                        dungeon[y][x] = 't'
                        break
                break


def add_exit(dungeon):
    valid = False
    while True:
        y = random.randint(0, 99)
        if ' ' in dungeon[y]:
            while True:
                x = random.randint(0, 99)
                if dungeon[y][x] == ' ':
                    dungeon[y][x] = '@'
                    return
            
        


import random

class Weapon():
    def __init__(self, name, damage, sharpness, weight):
        """Initiate the Weapon"""
        self.name = name
        self.damage = damage
        self.sharpness = sharpness
        self.weight = weight

        self.tier = int(((self.damage+self.sharpness+self.weight)*2)/10)


    def stats(self):
        return f"{self.name.upper()} DMG: {str(self.damage)} | SHRP: {str(self.sharpness)} | WGHT: {str(self.weight)} | TIER: {str(self.tier)}"



weapon_class = ['cracked', 'thorgan', "knight's", "orc's", "goblin's", "rusty", "old", "new"]

weapon_type = ["sword", "spear", "club", "mace", "morning star", "broad sword", "longsword", "dagger", "dirk", "shortsword", "sabre", "katana", "battle axe", "flail", "quarter staff", "war hammer", "pike", "poleaxe"]

def generate_weapon():
    """Generate a random weapon"""

    weapon = Weapon(
        name=random.choice(weapon_class) + " " + random.choice(weapon_type),
        damage = random.randint(1, 10),
        sharpness = random.randint(1, 10),
        weight = random.randint(1, 10)
    )

    return weapon


def ghoul_weapon():
    """Returns a Ghoul's Voulge. This weapon can only be looted from a ghoul and not found"""
    weapon = Weapon(
        name="ghoul's voulge",
        damage = random.randint(8,10),
        sharpness = random.randint(8,10),
        weight = random.randint(8,10),
    )
    
    return weapon


def weapon_from_dict(dict):
    """Builds a weapon from Dict"""
    weapon = Weapon(
        name = dict['name'],
        damage = dict['damage'],
        sharpness = dict['sharpness'],
        weight = dict['weight'],
    )

    return weapon


def weapons_from_dict(_list):
    """Builds a list of weapons from list of Dicts"""

    out = []


    for w in _list:
        out.append(weapon_from_dict(w))

    return out


#! /usr/bin/env python
# coding: utf-8

# generator-1.py, a simple python dungeon generator by
# James Spencer <jamessp [at] gmail.com>.

# To the extent possible under law, the person who associated CC0 with
# pathfinder.py has waived all copyright and related or neighboring rights
# to pathfinder.py.

# You should have received a copy of the CC0 legalcode along with this
# work. If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.


# Taken from http://www.roguebasin.com/index.php/A_Simple_Dungeon_Generator_for_Python_2_or_3

# Changes that I made

# A few minior tweaks/updates to the syntax to run in Python 3.6

# Changed the functions to return dungeon in a nested list format
# to suit my needs. 

# Added code to extend list past the player's view


from __future__ import print_function
import random

CHARACTER_TILES = {'stone': '.',
                   'floor': ' ',
                   'wall': '#'}


class Generator():
    def __init__(self, width=64, height=64, max_rooms=15, min_room_xy=5,
                 max_room_xy=10, rooms_overlap=False, random_connections=1,
                 random_spurs=3, tiles=CHARACTER_TILES):
        self.width = width
        self.height = height
        self.max_rooms = max_rooms
        self.min_room_xy = min_room_xy
        self.max_room_xy = max_room_xy
        self.rooms_overlap = rooms_overlap
        self.random_connections = random_connections
        self.random_spurs = random_spurs
        self.tiles = CHARACTER_TILES
        self.level = []
        self.room_list = []
        self.corridor_list = []
        self.tiles_level = []

    def gen_room(self):
        x, y, w, h = 0, 0, 0, 0

        w = random.randint(self.min_room_xy, self.max_room_xy)
        h = random.randint(self.min_room_xy, self.max_room_xy)
        x = random.randint(1, (self.width - w - 1))
        y = random.randint(1, (self.height - h - 1))

        return [x, y, w, h]

    def room_overlapping(self, room, room_list):
        x = room[0]
        y = room[1]
        w = room[2]
        h = room[3]

        for current_room in room_list:

            # The rectangles don't overlap if
            # one rectangle's minimum in some dimension
            # is greater than the other's maximum in
            # that dimension.

            if (x < (current_room[0] + current_room[2]) and
                current_room[0] < (x + w) and
                y < (current_room[1] + current_room[3]) and
                current_room[1] < (y + h)):

                return True

        return False


    def corridor_between_points(self, x1, y1, x2, y2, join_type='either'):
        if x1 == x2 and y1 == y2 or x1 == x2 or y1 == y2:
            return [(x1, y1), (x2, y2)]
        else:
            # 2 Corridors
            # NOTE: Never randomly choose a join that will go out of bounds
            # when the walls are added.
            join = None
            if join_type == 'either' and set([0, 1]).intersection(
                 set([x1, x2, y1, y2])):

                join = 'bottom'
            elif join_type == 'either' and set([self.width - 1,
                 self.width - 2]).intersection(set([x1, x2])) or set(
                 [self.height - 1, self.height - 2]).intersection(
                 set([y1, y2])):

                join = 'top'
            elif join_type == 'either':
                join = random.choice(['top', 'bottom'])
            else:
                join = join_type

            if join == 'top':
                return [(x1, y1), (x1, y2), (x2, y2)]
            elif join == 'bottom':
                return [(x1, y1), (x2, y1), (x2, y2)]

    def join_rooms(self, room_1, room_2, join_type='either'):
        # sort by the value of x
        sorted_room = [room_1, room_2]
        sorted_room.sort(key=lambda x_y: x_y[0])

        x1 = sorted_room[0][0]
        y1 = sorted_room[0][1]
        w1 = sorted_room[0][2]
        h1 = sorted_room[0][3]
        x1_2 = x1 + w1 - 1
        y1_2 = y1 + h1 - 1

        x2 = sorted_room[1][0]
        y2 = sorted_room[1][1]
        w2 = sorted_room[1][2]
        h2 = sorted_room[1][3]
        x2_2 = x2 + w2 - 1
        y2_2 = y2 + h2 - 1

        # overlapping on x
        if x1 < (x2 + w2) and x2 < (x1 + w1):
            jx1 = random.randint(x2, x1_2)
            jx2 = jx1
            tmp_y = [y1, y2, y1_2, y2_2]
            tmp_y.sort()
            jy1 = tmp_y[1] + 1
            jy2 = tmp_y[2] - 1

            corridors = self.corridor_between_points(jx1, jy1, jx2, jy2)
            self.corridor_list.append(corridors)

        # overlapping on y
        elif y1 < (y2 + h2) and y2 < (y1 + h1):
            if y2 > y1:
                jy1 = random.randint(y2, y1_2)
                jy2 = jy1
            else:
                jy1 = random.randint(y1, y2_2)
                jy2 = jy1
            tmp_x = [x1, x2, x1_2, x2_2]
            tmp_x.sort()
            jx1 = tmp_x[1] + 1
            jx2 = tmp_x[2] - 1

            corridors = self.corridor_between_points(jx1, jy1, jx2, jy2)
            self.corridor_list.append(corridors)

        # no overlap
        else:
            join = None
            if join_type == 'either':
                join = random.choice(['top', 'bottom'])
            else:
                join = join_type

            if join == 'top':
                if y2 > y1:
                    jx1 = x1_2 + 1
                    jy1 = random.randint(y1, y1_2)
                    jx2 = random.randint(x2, x2_2)
                    jy2 = y2 - 1
                    corridors = self.corridor_between_points(
                        jx1, jy1, jx2, jy2, 'bottom')
                    self.corridor_list.append(corridors)
                else:
                    jx1 = random.randint(x1, x1_2)
                    jy1 = y1 - 1
                    jx2 = x2 - 1
                    jy2 = random.randint(y2, y2_2)
                    corridors = self.corridor_between_points(
                        jx1, jy1, jx2, jy2, 'top')
                    self.corridor_list.append(corridors)

            elif join == 'bottom':
                if y2 > y1:
                    jx1 = random.randint(x1, x1_2)
                    jy1 = y1_2 + 1
                    jx2 = x2 - 1
                    jy2 = random.randint(y2, y2_2)
                    corridors = self.corridor_between_points(
                        jx1, jy1, jx2, jy2, 'top')
                    self.corridor_list.append(corridors)
                else:
                    jx1 = x1_2 + 1
                    jy1 = random.randint(y1, y1_2)
                    jx2 = random.randint(x2, x2_2)
                    jy2 = y2_2 + 1
                    corridors = self.corridor_between_points(
                        jx1, jy1, jx2, jy2, 'bottom')
                    self.corridor_list.append(corridors)


    def gen_level(self):

        # build an empty dungeon, blank the room and corridor lists
        for i in range(self.height):
            self.level.append(['stone'] * self.width)
        self.room_list = []
        self.corridor_list = []

        max_iters = self.max_rooms * 5

        for a in range(max_iters):
            tmp_room = self.gen_room()

            if self.rooms_overlap or not self.room_list:
                self.room_list.append(tmp_room)
            else:
                tmp_room = self.gen_room()
                tmp_room_list = self.room_list[:]

                if self.room_overlapping(tmp_room, tmp_room_list) is False:
                    self.room_list.append(tmp_room)

            if len(self.room_list) >= self.max_rooms:
                break

        # connect the rooms
        for a in range(len(self.room_list) - 1):
            self.join_rooms(self.room_list[a], self.room_list[a + 1])

        # do the random joins
        for a in range(self.random_connections):
            room_1 = self.room_list[random.randint(0, len(self.room_list) - 1)]
            room_2 = self.room_list[random.randint(0, len(self.room_list) - 1)]
            self.join_rooms(room_1, room_2)

        # do the spurs
        for a in range(self.random_spurs):
            room_1 = [random.randint(2, self.width - 2), random.randint(
                     2, self.height - 2), 1, 1]
            room_2 = self.room_list[random.randint(0, len(self.room_list) - 1)]
            self.join_rooms(room_1, room_2)

        # fill the map
        # paint rooms
        for room_num, room in enumerate(self.room_list):
            for b in range(room[2]):
                for c in range(room[3]):
                    self.level[room[1] + c][room[0] + b] = 'floor'

        # paint corridors
        for corridor in self.corridor_list:
            x1, y1 = corridor[0]
            x2, y2 = corridor[1]
            for width in range(abs(x1 - x2) + 1):
                for height in range(abs(y1 - y2) + 1):
                    self.level[min(y1, y2) + height][
                        min(x1, x2) + width] = 'floor'

            if len(corridor) == 3:
                x3, y3 = corridor[2]

                for width in range(abs(x2 - x3) + 1):
                    for height in range(abs(y2 - y3) + 1):
                        self.level[min(y2, y3) + height][
                            min(x2, x3) + width] = 'floor'

        # paint the walls
        for row in range(1, self.height - 1):
            for col in range(1, self.width - 1):
                if self.level[row][col] == 'floor':
                    if self.level[row - 1][col - 1] == 'stone':
                        self.level[row - 1][col - 1] = 'wall'

                    if self.level[row - 1][col] == 'stone':
                        self.level[row - 1][col] = 'wall'

                    if self.level[row - 1][col + 1] == 'stone':
                        self.level[row - 1][col + 1] = 'wall'

                    if self.level[row][col - 1] == 'stone':
                        self.level[row][col - 1] = 'wall'

                    if self.level[row][col + 1] == 'stone':
                        self.level[row][col + 1] = 'wall'

                    if self.level[row + 1][col - 1] == 'stone':
                        self.level[row + 1][col - 1] = 'wall'

                    if self.level[row + 1][col] == 'stone':
                        self.level[row + 1][col] = 'wall'

                    if self.level[row + 1][col + 1] == 'stone':
                        self.level[row + 1][col + 1] = 'wall'

    def gen_tiles_level(self):

        final = []

        for row_num, row in enumerate(self.level):
            tmp_tiles = []

            for col_num, col in enumerate(row):
                if col == 'stone':
                    tmp_tiles.append(self.tiles['stone'])
                if col == 'floor':
                    tmp_tiles.append(self.tiles['floor'])
                if col == 'wall':
                    tmp_tiles.append(self.tiles['wall'])

            final.append(tmp_tiles)
        

        return final


import random
from code import terminal
from people import chat


words = ['the...', 'the', 'peice', 'heart', 'magic', 'flow', 'river', 'stone', 'walls', 'ghouls', 'they', 'them', 'i...', 'i', 'who...']

class Grarlan():
    def __init__(self):
        self.symbol = 'G'

        self.voice = chat.Chat("Grarlan: ", "\033[34m")

    def speak(self, player):
        terminal.clear()
        t = []
        for i in range(random.randint(1, 8)):
            t.append(random.choice(words))

        self.voice.say(" ".join(t))






class Buff():
    def __init__(self, dmg, enemy_dmg,lasts, reason, des, enemy=False):
        """Initiate a Buff"""
        self.dmg = dmg
        self.enemy_dmg = enemy_dmg
        self.lasts = lasts+1 # to account for buff being removed after calling
        self.reason = reason
        self.des = des
        self.enemy = enemy


import json
from classes import weapon, armor, player, player_from_dict
import os


def initiate():
	if not os.path.isfile('save.txt'):
		with open('save.txt', 'x') as file:
			pass
	if not os.path.isfile('save_map.txt'):
		with open('save_map.txt', 'x') as file:
			pass



def class_to_json(object):
	"""Converts a 'nested' class to 'nested' JSON"""

	result = json.loads(json.dumps(object, default=lambda o: o.__dict__))

	return result



def get_game_data():
	"""Converts 'nested' JSON to 'nested' Game Data and Player Object"""

	with open('save.txt', 'r') as file:
		JSON = json.loads(file.read())

	player = player_from_dict.PlayerFromDict(JSON['player'])
	game_map = map_from_file(JSON['game_map'])
	posx = JSON['posx']
	posy = JSON['posy']

	return [player, game_map, posx, posy]




def save_player_data(player, _map, posx, posy):
	"""Writes Player Data to File"""

	JSON = {
		'player' : class_to_json(player),
		'game_map' : 'save_map.txt',
		'posx' : posx,
		'posy' : posy,
	}

	with open('save_map.txt', 'w') as file:
		file.write(map_to_string(_map))

	with open('save.txt', 'w') as file:
		file.write(json.dumps(JSON))




def map_to_string(_map):
	"""Converts a Nested List map to String"""
	string = ""

	for i in _map:
		row = ""
		for j in i:
			row += j
		row += '\n'
		string += row

	return string


def map_from_file(file):
	"""Converts a string to a nested list"""
	with open(file, 'r') as file:
		data = file.read()

	_map = []

	rows = data.split("\n")

	for row in rows:
		r = [char for char in row]
		_map.append(r)

	return _map




import random
from code import terminal
from people import chat


class Rohan():
    def __init__(self):
        self.symbol = 'R'

        self.voice = chat.Chat("Rohan: ", "\033[34m")

    def speak(self, player):
        terminal.clear()

        self.voice.say("I thought I'd never see you again!")
        player.voice.say("Me?")
        self.voice.say("Yes yes... I... ah never mind then")
        player.voice.say("What is it?")
        self.voice.say("*cackles*")
        player.voice.say("What's so funny?")
        self.voice.say("Nothing... nothing at all.")

        i = self.choice("[1] Ask him how he got here\n[2] Leave him alone", valid=['1', '2'])

        if i == '1':
            terminal.clear()
            self.voice.say("Crazy one ain't you. Dem were the breaks where I where been")
            player.voice.say("I don't understand...")
            self.voice.say("I saw it... It was there, just through the portal infront of me")
            player.voice.say("What was it?")
            self.voice.say("I was too weak. I fell to my knees, so close. Then it was gone")
            player.voice.say("What was gone?")
            self.voice.say('Gone gone gone... I saw and it now it is gone')
        else:
            terminal.clear()
            self.voice.say("The ghouls are close by... beware")
        
    def choice(self, prompt, valid):
        """Todo: Update to Navigate Function"""
        while True:
            print(prompt)
            i = input()
    
            if i in valid:
                return i




from people import chat
from classes import weapon
import random
from code import util, terminal

def calc_cost(w):
    """Calculates the cost of the weapon based on Tier"""

    _min = 12
    _max = 18

    if w.tier != 0:
        _min += random.randint(1, w.tier)*w.tier
        _max += random.randint(1, w.tier)*w.tier
    else:
        w = weapon.generate_weapon()
        self.calc_cost(w)


    if _min > _max:
        return random.randint(_max, _min)
    else:
        return random.randint(_min, _max)

class Maximus():
    def __init__(self):
        self.symbol = 'M'

        self.voice = chat.Chat("Maximus: ", "\033[34m")

        self.weapon1 = weapon.generate_weapon()
        self.weapon2 = weapon.generate_weapon()
        self.weapon3 = weapon.generate_weapon()

        self.weapon1_cost = calc_cost(self.weapon1)
        self.weapon2_cost = calc_cost(self.weapon2)
        self.weapon3_cost = calc_cost(self.weapon3)
        

    def speak(self, player):
        terminal.clear()
        if player.maximus_talked_to:
            self.voice.say("Eh... You again? Still haven't found your friend? Never mind... Looking for another trade?")
            i = self.choice("[1] Trade\n[2] Leave", valid=['1','2'])
            if i == '1':
                self.trade(player)
            else:
                self.voice.say("Good luck to you then!")
        else:
            player.maximus_talked_to = True
            self.voice.say("Hello there!")
            player.voice.say("Who are you?")
            self.voice.say("Me? Oh I'm Maximus Alchetren, and I was once a mighty king! Now I'm just a survivor here like the rest of us")
            player.voice.say("How long have you been here?")
            self.voice.say("Too long my friend, I cannot remember what the outside world even looks like now.")
            i = self.choice("[1] Ask him what he came here for\n[2] Ask him if he knows of a way out", valid=['1', '2'])
            if i == '1':
                player.voice.say("Why did you come here?")
                self.voice.say("Ah! A good question. I was power hungry, I wished to find the heart of magic and use it for my own")
                player.voice.say("The heart of magic?")
                self.voice.say("Isn't it what we are all here for? It is the very source of the river of magic, the ultimate power")
                i = self.choice("[1] Ask him if he has seen you before\n[2] Ask him if he knows a way out", valid=['1', '2'])
                if i == '1':
                    self.voice.say("You? I've seen you many a time, you don't remember me do you?")
                    player.voice.say('Not really')
                    self.voice.say("Hmm. Death's Dungeon has a strange way of working on the mind. Say, what happened to your friend?")
                    player.voice.say("Friend?")
                    self.voice.say("Yes, you had someone with you last time, eh... maybe he's dead")
                elif i == '2':
                    self.voice.say("A way out? I've tried so many times. This place is a maze, there is no finding your way out! I tried mining through the walls, but every time I hit rock so hard it is impossible to break, or rivers of lava")
                    player.voice.say("Has anyone made it out?")
                    self.voice.say("The only way to escape this place is by death. I have tried to stay sane and hopeful, I must admit I know I am trapped here forever, there is no way out.")
            else:
                self.voice.say("A way out? I've tried so many times. This place is a maze, there is no finding your way out! I tried mining through the walls, but every time I hit rock so hard it is impossible to break, or rivers of lava")
                player.voice.say("Has anyone made it out?")
                self.voice.say("The only way to escape this place is by death. I have tried to stay sane and hopeful, I must admit I know I am trapped here forever, there is no way out.")


    def choice(self, prompt, valid):
        """Todo: Update to Navigate Function"""
        while True:
            print(prompt)
            i = input()
    
            if i in valid:
                return i


    def trade(self, player):
        self.voice.say("What would you like to have a look at? My Weapons or my Supplies?")
        i = self.choice("[1] Weapons\n[2] Supplies", valid=['1', '2'])

        if i == '1':
            print(f"[1] {self.weapon1.name} DMG: {str(self.weapon1.damage)} | SHRP: {str(self.weapon1.sharpness)} | WGHT: {str(self.weapon1.weight)} TIER: {str(self.weapon1.tier)} --- COST: {str(self.weapon1_cost)}")
            print(f"[2] {self.weapon2.name} DMG: {str(self.weapon2.damage)} | SHRP: {str(self.weapon2.sharpness)} | WGHT: {str(self.weapon2.weight)} TIER: {str(self.weapon2.tier)} --- COST: {str(self.weapon2_cost)}")
            print(f"[3] {self.weapon3.name} DMG: {str(self.weapon3.damage)} | SHRP: {str(self.weapon3.sharpness)} | WGHT: {str(self.weapon3.weight)} TIER: {str(self.weapon3.tier)} --- COST: {str(self.weapon3_cost)}")
            print("[4] Exit")

            i = util.get_input("-> ", valid=['1', '2', '3', '4'])

            if i == '1':
                self.purchase_weapon(self.weapon1, self.weapon1_cost, player)
            elif i == '2':
                self.purchase_weapon(self.weapon2, self.weapon2_cost, player)
            elif i == '3':
                self.purchase_weapon(self.weapon3, self.weapon3_cost, player)
            elif i == '4':
                self.voice.say("Suit yourself...")
                terminal.clear()
        else:
            print("[1] Apples -- 5 silver")
            print("[2] Bread  -- 15 silver")
            print("[3] Severed Hand -- 3 Gold")
            print("[4] Exit")
            
            choice = util.get_input("-> ", valid=['1', '2', '3','4'])

            if choice == '1' and player.silver >= 5:
                player.silver -= 5
                player.inventory.append("apple")
            elif choice == '2' and player.silver >= 15:
                player.silver -= 15
                player.inventory.append("bread")
            elif choice == '3' and player.gold >= 3:
                player.gold -= 3
                player.inventory.append("severed hand")
            elif choice == '4':
                pass
            else:
                self.voice.say("Hey! Are you trying to cheat me!")
                return
            self.voice.say("Pleasure doing business with you.")






    def purchase_weapon(self, weapon, cost, player):
        if player.gold >= cost:
            print("Purchased Weapon")
            player.gold -= cost
            player.store_weapon(weapon)

        else:
            self.voice.say("Hey! Are you trying to stiff me! You don't have enough gold for that!")
        


from code import terminal


def get_input(prompt, valid):
    while True:
        i = input(prompt)

        if i in valid:
            return i
        else:
            print("Not a valid input!\n")


def floor(val):
    """Converts any Negative number to 0"""
    if val < 0:
        return 0
    else:
        return val



def center(string, max_length):
    if len(string) >= max_length:
        print(string)
    else:
        needed_chars = max_length-len(string)
        if needed_chars % 2 == 0:
            size = " "*int(needed_chars/2)
            print(size+string+size)
        else:
            size = " "*int(needed_chars/2)
            print(size+" "+string+size)


def indent(string, size):
    t = " "*int(size)
    print(t+string)

def cont():
    input("\n\nPress Enter to Continue")
    terminal.clear()



def log(string):
    with open('log.txt', 'a+') as file:
        file.write("\n"+string)


def clear_log():
    with open('log.txt', 'w') as file:
        file.write("")


from classes import weapon, armor
import random
from code import util, terminal
from battle import buff, enemy

#############
#  GOBLINS  #
#############

def generate_goblin():
    name = "goblin"
    life = random.randint(10, 20)
    magic = 0
    can_magic = False
    w = weapon.generate_weapon()
    a = armor.generate_armor()

    silver = random.randint(1, 5)
    gold = 0
    diamonds = 0


    can_heal = False

    b = buff.Buff(0, 10, 1, "Cobalorum Unify", "Goblins Do +10 Damage")

    goblin = enemy.Enemy(name, life, magic, w, a, can_magic, can_heal, silver, gold, diamonds, b)

    return goblin

##########
#  ORCS  #
##########

def generate_orc():
    name = "orc"
    life = random.randint(50, 100)
    magic = 0
    can_magic = False
    w = weapon.generate_weapon()
    a = armor.generate_armor()

    silver = random.randint(10, 20)
    gold = random.randint(5, 8)
    diamonds = 0

    can_heal = False

    b = buff.Buff(-10, 0, 3, "debilitatem fluxus", "-10 Damage on Your Attacks")

    orc = enemy.Enemy(name, life, magic, w, a, can_magic, can_heal, silver, gold, diamonds, b)

    return orc

##############
#  THORGANS  #
##############

def generate_thorgan():
    name = "thorgan"
    life = random.randint(150, 500)
    magic = 0
    can_magic = False
    w = weapon.generate_weapon()
    a = armor.generate_armor()

    silver = random.randint(50, 90)
    gold = random.randint(10, 30)
    diamonds = random.randint(1, 6)

    can_heal = False

    b = None

    thorgan = enemy.Enemy(name, life, magic, w, a, can_magic, can_heal, silver, gold, diamonds, b)

    return thorgan

############
#  VITTRA  #
############

# note: only invisible enemy

def generate_vittra():
    name = "vittra"
    life = random.randint(5, 10)
    magic = 30
    can_magic = True
    w = weapon.generate_weapon()
    a = armor.generate_armor()

    silver = random.randint(10, 20)
    gold = random.randint(5, 10)
    diamonds = random.randint(0, 1)

    can_heal = True

    b = None

    vittra = enemy.Enemy(name, life, magic, w, a, can_magic, can_heal, silver, gold, diamonds, b)

    return vittra


#############
#  WYBBERS  #
#############

def generate_wybber():
    name = "wybber"
    life = random.randint(35, 60)
    magic = 15
    can_magic = True
    w = weapon.generate_weapon()
    a = armor.generate_armor()

    silver = 0
    gold = 0
    diamonds = random.randint(3, 5)

    can_heal = True

    b = buff.Buff(0, 100, 1, "Exponentia Instantis Mortis", "+100 Damage")

    wybber = enemy.Enemy(name, life, magic, w, a, can_magic, can_heal, silver, gold, diamonds, b)

    return wybber


############
#  GHOULS  #
############

def generate_ghoul():
    name = "ghoul"
    life = 100
    magic = 0
    can_magic = False
    w = weapon.ghoul_weapon()
    a = armor.ghoul_armor()

    silver = 0
    gold = 0
    diamonds = 0

    can_heal = False

    b = buff.Buff(-30, 0, 1, "Gelida Cogitationes", "-30 Damage on Your Attacks")

    ghoul = enemy.Enemy(name, life, magic, w, a, can_magic, can_heal, silver, gold, diamonds, b)

    return ghoul



def generate_monsters():
    i = random.randint(1, 20)
    count = random.randint(1, 3)

    monsters = []

    if i in range(1, 5):
        for i in range(count):
            monsters.append(generate_goblin())
    elif i in range(6, 10):
        for i in range(count):
            monsters.append(generate_orc())
    elif i in range(11, 13):
        for i in range(count):
            monsters.append(generate_vittra())
    elif i == 14:
        monsters.append(generate_thorgan())
    elif i in range(15, 18):
        monsters.append(generate_wybber())
    elif i == 19:
        monsters.append(generate_ghoul())
    else:
        for i in range(count):
            monsters.append(generate_ghoul())
    
    return monsters
        
        



import sys, os, time
from code import terminal


class Chat():
    def __init__(self, name, color):
        self.name = name
        self.color = color

        self.reset = "\033[0m"


    def say(self, text):
        print(self.color+self.name+self.reset, end='')
        for i in text:
            sys.stdout.write(i)
            sys.stdout.flush()
            time.sleep(0.004)
        print()
        input()




import sys, os, time


def clear():
    """Clears the screen"""
    os.system('clear')


def custom(fg, bg=[]):
    """Builds a custom color from an RGB Value"""
    if bg:
        return f"\033[38;2;{fg[0]};{fg[1]};{fg[2]}m\033[48;2;{self.bg[0]};{self.bg[1]};{self.bg[2]}m"
    else:
        return f"\033[38;2;{fg[0]};{fg[1]};{fg[2]}m"


def slow_print(string, t=0.05):
    """Prints a string letter by letter"""
    for letter in string:
        sys.stdout.write(letter)
        sys.stdout.flush()
        time.sleep(t)
    print()
    


    

reset = "\033[0m"
fgBlack = "\033[30m"
fgBrightBlack = "\033[30;1m"
bgBlack = "\033[40m"
bgBrightBlack = "\033[40;1m"
fgRed = "\033[31m"
fgBrightRed = "\033[31;1m"
bgRed = "\033[41m"
bgBrightRed = "\033[41;1m"
fgGreen = "\033[32m"
fgBrightGreen = "\033[32;1m"
bgGreen = "\033[42m"
bgBrightGreen = "\033[42;1m"
fgYellow = "\033[33m"
fgBrightYellow = "\033[33;1m"
bgYellow = "\033[43m"
bgBrightYellow = "\033[43;1m"
fgBlue = "\033[34m"
fgBrightBlue = "\033[34;1m"
bgBlue = "\033[44m"
bgBrightBlue = "\033[44;1m"
fgMagenta = "\033[35m"
fgBrightMagenta = "\033[35;1m"
bgMagenta = "\033[45m"
bgBrightMagenta = "\033[45;1m"
fgCyan = "\033[36m"
fgBrightCyan = "\033[36;1m"
bgCyan = "\033[46m"
bgBrightCyan = "\033[46;1m"
fgWhite = "\033[37m"
fgBrightWhite = "\033[37;1m"
bgWhite = "\033[47m"
bgBrightWhite = "\033[47;1m"


from classes import armor, weapon
from code import util, terminal, randomness
from people import chat
import random

class PlayerFromDict():
    def __init__(self, dict):
        """Initiate the Player from Dict"""

        # Life/Magic
        # EDIT: Stamina has been Removed
        self.life = dict['life']
        self.max_life = dict['max_life']
        self.magic = dict['magic']
        self.max_magic = dict['max_magic']

        # Weapons/Armor
        self.weapon = weapon.weapon_from_dict(dict['weapon'])
        self.armor = armor.armor_from_dict(dict['armor'])

        # Inventory/Money
        self.inventory = dict['inventory']
        self.max_weapons = dict['max_weapons']
        self.max_armors = dict['max_armors']
        self.weapons = weapon.weapons_from_dict(dict['weapons'])
        self.armors = armor.armors_from_dict(dict['armors'])
        self.gold = dict['gold']
        self.silver = dict['silver']
        self.diamonds = dict['diamonds']
        self.shards = dict['shards']

        # voice
        self.voice = chat.Chat("You: ", "\033[37m")

        #Keep track of rooms played
        self.rooms_played = dict['rooms_played']

        #Keep Track of Characters
        self.trekker_alive = dict['trekker_alive']
        self.maximus_talked_to = dict['maximus_talked_to']
        self.laurence_talked_to = dict['laurence_talked_to']
        self.warren_talked_to = dict['warren_talked_to']

    def attack(self, enemy, buffs, enemies, noloot=False):
        """Attack the Enemy and add Effects for Buffs"""

        terminal.clear()

        green = "\033[32m"
        red = "\033[31m"
        reset = "\033[0m"

        raw_dmg = (self.weapon.damage+self.weapon.sharpness)*self.weapon.weight

        terminal.slow_print(f"{green}+{str(raw_dmg)}{reset}")


        dmg = (self.weapon.damage+util.floor(self.weapon.sharpness - enemy.armor.armor))*self.weapon.weight

        armor_block = raw_dmg-dmg

        terminal.slow_print(f"{red}   -{str(armor_block)} -- armor{reset}")

        for buff in buffs:
            if buff.dmg:
                dmg += buff.dmg
                if buff.dmg > 0:
                    terminal.slow_print(f"{green}   +{buff.dmg} --{buff.reason}{reset}")
                else:
                    terminal.slow_print(f"{red}   -{buff.dmg} --{buff.reason}{reset}")

        if dmg > 0:
            terminal.slow_print(f" {green}{str(dmg)} Damage{reset}")
        else:
            dmg = 0
            terminal.slow_print(f" {red}{str(dmg)} Damage{reset}")

        enemy.life -= dmg

        if enemy.life <= 0:
            print("You defeated the " + enemy.name)
            enemies.remove(enemy)
            if noloot:
                return
            else:
                self.loot(enemy)


            

    def loot(self, enemy):
        """Loot the Enemy"""
        if enemy.silver:
            print(f"You looted {str(enemy.silver)} silver")
            
        if enemy.gold:
            print(f"You looted {str(enemy.gold)} gold")
            
        if enemy.diamonds:
            print(f"You looted {str(enemy.diamonds)} diamonds")

        self.silver += enemy.silver
        self.gold += enemy.gold
        self.diamonds += enemy.diamonds

        magic = random.randint(1, 3)

        print(f"You gained {str(magic)} magic")

        self.magic += magic
        if self.magic > self.max_magic:
            self.magic = self.max_magic

        print(enemy.weapon.stats())

        print("[1] Keep")
        print("[2] Throw Away")
        
        choice = util.get_input("-> ", valid=['1','2'])

        if choice == '1':
            self.store_weapon(enemy.weapon)
        else:
            print('You threw it away')

        print(enemy.armor.stats())

        print("[1] Keep")
        print("[2] Throw Aaway")

        choice = util.get_input("-> ", valid=['1', '2'])

        if choice == '1':
            self.store_armor(enemy.armor)
        else:
            print("You threw it away")
        
                
            

        

    
    def get_stats(self):
        """Return Basic Player Stats"""
        return f"LIFE: {str(self.life)}/{str(self.max_life)} | MAGIC: {str(self.magic)}/{str(self.max_magic)}"


    def eat(self):
        """Eat Food to Heal"""
        while True:
            terminal.clear()
            print(self.get_stats())

            print("\n\n")
    
            items = []
    
            for item in sorted(self.inventory):
                exist = False
                for i in items:
                    if item == i[0]:
                        i[1] += 1
                        exist = True
                if not exist:
                    items.append([item, 1])
    
            for item in items:
                print(f"{item[0]} x{str(item[1])}")
    
            print("\nChoose an item to eat")
    
            choice = input("-> ").lower()

            if choice == 'exit':
                return
    
            if choice in self.inventory:
                found = False
                for food in randomness.foods:
                    if food['name'].lower() == choice:
                        print("You ate a " + choice)
                        print("+"+str(food['health']) + " health")
                        self.life += food['health']
                        if self.life > self.max_life:
                            self.life = self.max_life
                        found = True
                        self.inventory.remove(choice)
                if not found:
                    print(choice+" is not a food!")
    
            input("\n\nPress Enter to Continue")

    def shuffle_loadout(self):
        """Shuffle Weapon and Armor Loadout"""
        while True:
            terminal.clear()
    
            print("#### CURRENT LOADOUT ####\n")
            print(f"\nWEAPON: {self.weapon.name.capitalize()} DMG: {str(self.weapon.damage)} | SHRP: {str(self.weapon.sharpness)} | WGHT: {str(self.weapon.weight)} | TIER: {str(self.weapon.tier)}")
            print(f"\nARMOR: {self.armor.name.capitalize()} ARMOR: {str(self.armor.armor)} | TIER: {str(self.armor.tier)}")
    
            print("\n\n")
    
            print("[1] Equip Weapon")
            print("[2] Equip Armor")
            print("[3] Exit")
    
            choice = util.get_input('-> ', valid=['1', '2', '3'])
    
            if choice == '1':
                valids = []
                for w in self.weapons:
                    index = str(self.weapons.index(w))
                    valids.append(index)
                    print(f"[{index}] {w.name} DMG: {str(w.damage)} | SHRP: {str(w.sharpness)} | WGHT: {str(w.weight)} | TIER: {str(w.tier)}")
    
                index = int(util.get_input("-> ", valid=valids))
    
                self.equip_weapon(self.weapons[index])
    
            elif choice == '2':
                valids = []
                for a in self.armors:
                    index = str(self.armors.index(a))
                    valids.append(index)
                    print(f"[{index}] {a.name} ARMOR: {str(a.armor)} | TIER: {str(a.tier)}")
    
                index = int(util.get_input("-> ", valid=valids))
    
                self.equip_armor(self.armors[index])
    
            else:
                break

            input("Press Enter to Continue")

            
        

    def equip_weapon(self, weapon):
        """Equip A Weapon"""
        self.weapon = weapon
        print(f"{self.weapon.name} equiped")

    def store_weapon(self, weapon):
        """Store A Weapon"""
        if len(self.weapons) >= 3:
            print("Not enough space to store weapon!")
            print("[1] Throw away")
            print("[2] Replace a different weapon")

            choice = util.get_input("-> ", valid=['1', '2'])

            if choice == '1':
                print("You threw the " + weapon.name + " away")

            else:
                print("Choose a weapon to be replaced: \n")
                
                valids = []
                for w in self.weapons:
                    index = str(self.weapons.index(w))
                    valids.append(index)
                    print(f"[{index}] {w.name} DMG: {str(w.damage)} | SHRP: {str(w.sharpness)} | WGHT: {str(w.weight)} | TIER: {str(w.tier)}")

                choice = util.get_input("-> ", valid=valids)

                self.weapons[int(choice)] = weapon

                print("Weapon stored.")
        else:
            self.weapons.append(weapon)
            print("Weapon stored.")


    def equip_armor(self, armor):
        """Equip Armor"""
        self.armor = armor
        print(f"{self.armor.name} equiped")

    def store_armor(self, armor):
        """Store Armor"""
        if len(self.armors) >= 3:
            print("Not enough space to store armor!")
            print("[1] Throw away")
            print("[2] Replace a different armor")

            choice = util.get_input("-> ", valid=['1', '2'])

            if choice == '1':
                print("You threw the " + armor.name + " away")

            else:
                print("Choose armor to be replaced: \n")
                
                valids = []
                for a in self.armors:
                    index = str(self.armors.index(a))
                    valids.append(index)
                    print(f"[{index}] {a.name} ARMOR: {str(a.armor)} | TIER: {str(a.tier)}")

                choice = util.get_input("-> ", valid=valids)

                self.armors[int(choice)] = armor
                print("Armor stored.")
        else:
            self.armors.append(armor)
            print("Armor stored.")
    
    def remember_convo1(self):
        """Remember the Conversation you had with Trekker"""

        trekker = chat.Chat("Trekker: ", "\033[34m")

        terminal.slow_print("The shard pulsates in light and a vision flashes infront of your eyes")

        input()

        terminal.clear()

        self.voice.say("How will you find it Trekker? Even with your abilities it will be nearly impossible!")
        trekker.say("Calm down Mikkel, remember the shards")
        self.voice.say("Shards? I though Ferric was just joking around")
        trekker.say("He wasn't, I found one today. I can follow the trail, I'm sure of it")
        self.voice.say("It is too dangerous, we must stay and gaurd the gate")
        trekker.say("We are all grown men here, when we swore our oath to protect this place we knew how dangerous it would be. We cannot just sit and guard the gate, we have to go on the offensive")
        self.voice.say("It's too dangerous... I can't as the leader let you do it")
        trekker.say("Mikkel! Listen to yourself! Do you seriously think we can stay here and guard it forever? Even if our bodies do not grow older our minds will. Ferric himself has admitted the spell will not last forever, it is already decaying! Something has to be done!")
        self.voice.say("And you think traveling into Death's very own dungeon is what we have to do")
        trekker.say("It's our only option.")
        self.voice.say("We don't know that, there is always another way, we can recruit successors for the guard")
        trekker.say("That isn't going to work, death grows more powerful each day, we cannot let it continue. We must do something!")
        self.voice.say("I cannot argue with you Trekker, I know you are correct. ")
        trekker.say("Me, Warren and William will go. That will leave you Ferric and Coyote to guard the portal. If we do not return, don't come after us")
        self.voice.say("Good luck Trekker.")

        terminal.clear()

        terminal.slow_print("The stone's light fades and the voices grow quieter and quieter")

        util.cont()

        terminal.clear()

        terminal.slow_print("For some reason you feel stronger and more powerful now, as if the shard has made your strength grow!")

    def remember_convo2(self):
        """Remember your Second Conversation with Trekker"""

        util.cont()

        terminal.clear()

        trekker = chat.Chat("Trekker: ", "\033[34m")

        trekker.say("Mikkel!")
        self.voice.say("What is it Trekker? Is something wrong?")
        trekker.say("Yes... horribly the wrong")
        self.voice.say("What has happened?")
        trekker.say("The king attacked our supporters in the mortal realm, all of them are dead")
        self.voice.say("Warrens-")
        trekker.say("Yes, his son. Ferric's brother, all of them. What are we going to do?")
        self.voice.say("Guard the gate. I will return")

        util.cont()

        terminal.clear()
                    

    def remember_convo3(self):
        """Warren Gives you the Letters for his son"""

        util.cont()

        terminal.clear()


        warren = chat.Chat("Warren: ", "\033[34m")


        warren.say("Take these, I cannot leave the gate, if I do I will not have the willpower to return to my post")
        self.voice.say("What is it-")
        warren.say("My letters for my son. I wanted to send them a long time ago... but I... I was a little afraid he wouldn't even want to read them")
        self.voice.say("Warren i-")
        warren.say("Just take them to him, please. Swear to me you will take them to him")
        self.voice.say("I never told you about... ")
        warren.say('About what?')
        self.voice.say("Never mind. I give you my word I will take these two your son")
        warren.say('Thankyou Mikkel, I trust you will')

        util.cont()

        terminal.clear()
    

    def remember_convo4(self):
        """Coyote and Ferric go into Deaths Dungeon"""

        util.cont()

        terminal.clear()

        coyote = chat.Chat("Coyote: ", "\033[34m")
        ferric = chat.Chat("Ferric: ", "\033[42m")


        coyote.say("We have to go after them. We cannot just let them perish")

        self.voice.say("We cannot abandon our post")

        coyote.say("I'm not saying that we should. I can stay here and guard the gate. You and Ferric go save them")

        ferric.say("You are a mighty warrior Coyote, but also the smallest and weakest among us, you would be overwhelmed and defeated if the Ghouls attacked")

        coyote.say("I'm faster then both of you, that makes up for my lack of size and strength")

        self.voice.say("I will stay. Coyote you know Trekker best, if anyone can find him it will be you. And Ferric, I barely count as a wizard, if they are wounded or hurt, they will need your healing magic")

        coyote.say("Are you sure Mikkel?")

        self.voice.say("My size and weight would only make fighting more difficult in that dungeon maze. Both of you should go. I will protect the gate till you return")

        ferric.say("And if we don't?")

        self.voice.say("If you do not return, then all hope is lost. We can only protect the mortal realm for so long, we must find the heart of magic")

        util.cont()

        terminal.clear()

    



from code import terminal, util, randomness
from battle import die, buff
import random


def living_enemies(enemies):
    """Check if any enemies are still alive"""
    val = False
    for enemy in enemies:
        print(enemy)
        if enemy.life > 0:
            val = True

    return val


def fight(player, enemies):
    """Fight between Player and Enemy"""

    player_turn = True # player always start, might switch to player_turn = random.randint(0, 1)

    valid_nums = [str(i) for i in range(0, 9)]

    buffs = []

    last_hit = None

    green = "\033[32m"
    red = "\033[31m"
    reset = "\033[0m"

    ########################
    # DON'T FORGET TO USE  #
    # MINIMAL BATTLE STYLE #
    ########################

    while player.life > 0 and living_enemies(enemies) > 0:
        while player_turn:
            terminal.clear()
            print(player.get_stats())
            print("\n\n---------- BUFFS -----------\n")
            for b in buffs:
                if not b.enemy:
                    print(f"{red}{b.reason} -> {b.des}{reset}")
                else:
                    print(f"{green}{b.reason} -> {b.des}{reset}")
            if not buffs:
                print("None")
            print("\n------------------------------")
            print("\n\n---------- ENEMIES -----------\n")
            for enemy in enemies:
                print(f"[{str(enemies.index(enemy))}] {enemy.name}-> {enemy.get_battle_stats()}")
            print("\n------------------------------")

            commands = ['?', 'attack', 'loadout', 'drink', 'magicae', 'run']
            
            full = input("-> ")

            c = full.split()

            command = c[0]

            if command in commands:
                # use try to invalid commands
                if command == '?':
                    print("Figure it out loser!")
                
                elif command == 'attack':
                    target = c[1]

                    if target in valid_nums:
                        enemy = enemies[int(target)]
                        player.attack(enemy, buffs, enemies)
                        last_hit = enemy
                        player_turn = False
                    else:
                        print("No enemy found!")
                
                elif command == 'loadout':
                    player.shuffle_loadout()
                
                elif command == 'run':
                    if random.randint(1, 4) == 1:
                        die.die("Struck in the back while running like a coward.")
                    else:
                        print("You flee the battle")
                        return
                
                elif command == 'drink':
                    potion = " ".join(c[1:]).lower()

                    if potion in player.inventory and potion in randomness.valid_potions:
                        player.inventory.remove(potion)
                        
                        if potion == 'potion of energy':
                            b = buff.Buff(50, 0, 1, "Potentia Navitas", "+50 on next attack")
                            print("Buff Activated: +50 on next attack")
                            buffs.append(b)
                        elif potion == 'healing potion':
                            print("You drank the healing potion...")
                            player.life = player.max_life
                        player_turn = False
                    else:
                        print("You don't have that potion!")
                
                elif command == 'magicae':
                    spell = " ".join(c[1:])
                    if spell in randomness.valid_spells:
                        if spell == 'sana ego':
                            if player.magic >= 4:
                                player.magic -= 4
                                player.life += 10
                                if player.life > player.max_life:
                                    player.life = player.max_life
                            else:
                                print("You do not have enough power to throw that spell")

                        elif spell == 'sana vulnera':
                            if player.magic >= 5:
                                player.magic -= 5
                                player.life += 15
                                if player.life > player.max_life:
                                    player.life = player.max_life
                            else:
                                print("You do not have enough power to throw that spell")
                        
                        elif spell == 'sana omnis vulnera':
                            if player.magic >= 10:
                                player.magic -= 10
                                player.life += 40
                                if player.life > player.max_life:
                                    player.life = player.max_life
                            else:
                                print("You do not have enough power to throw that spell")
                        
                        elif spell == 'confirma impetum':
                            if player.magic >= 10:
                                player.magic -= 10
                                b = buff.Buff(40, 0, 1, "Confirma Impetum", "+40 Damage on Next Attack")
                                buffs.append(b)
                                print("Buff Activated: +40 Damage on Next Attack")
                            else:
                                print("You do not have enough power to throw that spell")
                        
                        elif spell == 'confirma defensio':
                            if player.magic >= 10:
                                player.magic -= 10
                                b = buff.Buff(0, -40, 1, "Confirma Defensio", "-40 Damage On Next Enemy Attack")
                                buffs.append(b)
                                print("Buff Activated: -40 Damage on Next Enemy Attack")
                            else:
                                print("You do not have enough power to throw that spell")
                        
                        elif spell == 'noxa hostilis':
                            if player.magic >= 25:
                                player.magic -= 25
                                print("All Enemies Recieve 50 Damage")
                                for enemy in enemies:
                                    enemy.life -= 50
                                    if enemy.life <= 0:
                                        enemies.remove(enemy)
                                        player.loot(enemy)
                            else:
                                print("You do not have enough power to throw that spell")
                    else:
                        print("Your strange magic words are confusing...")
                        print(spell + " is not a spell")
            util.cont()

        

        while not player_turn:
            player_turn = True
            for enemy in enemies:
                while True:
                    if enemy.life <= (player.weapon.damage+util.floor(player.weapon.sharpness-enemy.armor.armor))*player.weapon.weight:
                        if enemy.can_heal:
                            # try to save them selves
                            enemy.heal(player, player_turn, buffs, enemies)
                        else:
                            # else try to kill the player as they cannot heal
                            if last_hit == enemy:
                                enemy.desperate_attack(player)
                            else:
                                enemy.choose_attack(player, player_turn, buffs, enemies)
                        break
                    else:
                        # pick the attack based on on player, buffs for fellow enemies, current buffs against them, player life and possible damage they might deal to the player
                        t = enemy.choose_attack(player, player_turn, buffs, enemies)
                        if t == 1:
                            # Enemy Plays turn again
                            pass
                        elif t == 2:
                            # all enemies play again
                            player_turn = False
                        else:
                            break

        for b in buffs:
            b.lasts -= 1
            if b.lasts <= 0:
                buffs.remove(b)

    if player.life <= 0:
        die.die("slaughtered in battle")
                            




loot = [
    {"type":"money", "name":"gold", "max":10},
    {"type":"money", "name":"silver", "max":20},
    {"type":"money", "name":"diamond", "max":3},
    {"type":"inventory", "name":"healing potion", "max":1},
    {"type":"inventory", "name":"dry bones", "max":1},
    {"type":"inventory", "name":"potion of energy", "max":1},
    {"type":"inventory", "name":"moldy apple", "max":1},
    {"type":"inventory", "name":"old potato", "max":1},
    {"type":"inventory", "name":"withered carrot", "max":1},
    {"type":"inventory", "name":"severed hand", "max":1},
    {"type":"inventory", "name":"severed finger", "max":1},
    {"type":"inventory", "name":"rotting flesh", "max":1},
    {"type":"inventory", "name":"moldy bread", "max":1},
    {"type":"inventory", "name":"squishy tomato", "max":1},
    {"type":"inventory", "name":"damp onion", "max":1},
    {"type":"inventory", "name":"onion", "max":1},
    {"type":"inventory", "name":"bread", "max":1},
    {"type":"inventory", "name":"apple", "max":1},
    {"type":"armor", "name":"", "max":1},
    {"type":"weapon", "name":"", "max":1},
]


foods = [
    {'name':'dry bones', 'health':2},
    {'name':'moldy apple', 'health':10},
    {'name':'old potato', 'health':10},
    {'name':'withered carrot', 'health':6},
    {'name':'severed hand', 'health':30},
    {'name':'severed finger', 'health':16},
    {'name':'rotting flesh', 'health':-6},
    {'name':'squishy tomato', 'health':8},
    {'name':'damp onion', 'health':6},
    {'name':'onion', 'health':12},
    {'name':'bread', 'health':20},
    {'name':'moldy bread', 'health':16},
    {'name':'apple', 'health':15},
]

valid_potions = ['potion of energy', 'healing potion']

valid_spells = ['sana ego', 'sana vulnera', 'sana omnis ego', 'confirma impetum', 'confirma defensio', 'noxa hostilis']


import random

class Armor():
    def __init__(self, name, armor):
        """Initiate the Armor"""
        self.name = name
        self.armor = armor

        # Note: Removed Magic and Fire Armor stats
        # becuase they have little bearing on
        # real armor stats. 

        self.tier = int((self.armor*6)/10)

    def stats(self):
        return f"{self.name.upper()} STRENGTH: {str(self.armor)} | TIER: {str(self.tier)}"



armor_class = ['cracked', 'thorgan', "knight's", "orc's", "goblin's", "rusty", "old", "new", "leather", "iron", "chainmail", "scale", "boiled leather", "mail", "plated mail", "plate"]


def generate_armor():
    """Generate a random set of armor"""

    armor = Armor(
        name=random.choice(armor_class) + " armor",
        armor = random.randint(1, 10),
    )

    return armor


def ghoul_armor():
    """Returns a ghouls armor. Can only be looted and not found"""

    armor = Armor(
        name="ghoul's purge",
        armor=random.randint(5, 10),
    )

    return armor


def armor_from_dict(dict):
    """Builds Armor from Dict"""
    armor = Armor(
        name = dict['name'],
        armor = dict['armor']
    )
    return armor

def armors_from_dict(list):
    """Builds Armor from List of Dict"""

    out = []

    for a in list:
        out.append(armor_from_dict(a))

    return out



import random
from code import terminal
from people import chat


class Warren():
    def __init__(self):
        self.symbol = 'W'
        
        self.voice = chat.Chat("Warren: ", "\033[34m")

    def speak(self, player):
        terminal.clear()
        if not player.warren_talked_to:   
            player.warren_talked_to = True

            self.voice.say("Well well well, if it isn't good old Mikkel")
            player.voice.say("Warren? Is that... You?")
            self.voice.say("What's the matter Mikkel? Forgot about me already?")
            player.voice.say("I can't seem to remember a lot of things")
            self.voice.say("I'm not surprised. I hit you hard enough with that mace I thought your brains were going to be splattered on the wall")
            player.voice.say("I don't remember fighting you... I thought we were freinds")
            self.voice.say("Yeah, we were friends, we aren't any more, so turn around before I kill you for real this time")
            player.voice.say("I don't want to fight you Warren. I just want to find the heart of magic and get out of here!")
            self.voice.say("LIAR! You just want the power for yourself!")
            player.voice.say("We can find it together")
            self.voice.say("I don't think you realize exactly what is going on here. The heart of magic can only be used for a single wish. And I'm planning on using it")
            player.voice.say("Don't we want the same thing? To destroy Death and escape this place?")
            self.voice.say("Destroy Death? Your more insane then I am. Death and Life are two things that must always exist in the universe. You can't make death die, he is the very definition of dying and thus cannot be killed by it. Even if you could make him vanish, he is a universial being and will only be reincarnated the second someone anywhere in the universe dies")
            player.voice.say("Then I can destroy the gate, that way he will never be able to enter the mortal realm")
            self.voice.say("And we would be trapped here forever! Are you forgetting about that")
            player.voice.say("A sacrifice must be made somewhere")
            self.voice.say("... Right. A sacrifice. I suppose Ferric was just a sacrifice, Terkker was just a sacrifice... coyote william all of them... just a sacrifice weren't they?")
            player.voice.say("I don't know. I can't remember...")
            
            terminal.clear()

            self.voice.say("How about I remind you then? Yeah, I'll tell you what happened to them. They all came here, here to find that blasted thing and stop death. Then they all died.")
            player.voice.say("Trekker was alive... I spoke with him")
            self.voice.say("Was? I'm assuming he's dead now. Did you kill him like you did William?")
            player.voice.say("William? I didn't kill him!")
            self.voice.say("Yes you did Mikkel. You attacked us, you were raving mad, screaming garbage. William tried to help you, he was trying to heal you. And what did you do?")
            player.voice.say("I... I don't remember")
            self.voice.say("You killed him. Stabbed him through the heart.")
            player.voice.say("And Coyoote? Did I kill him?")
            self.voice.say("No no, I did that. That scrappy little scout got in my way. I told him to step aside, being the idiot he was he wouldn't, so I killed him.")
            player.voice.say("Coyote was our brother")
            self.voice.say("So was William, and Ferric, who died trying to escape")
            player.voice.say("No! I didn't kill William, I can't have")
            self.voice.say("Heh. Your right. I killed him")
            player.voice.say("You what?")
            self.voice.say("You and William teamed up against me. Becuase I didn't want to use that stupid stone to destroy death, I want to use it to bring my family back, and that's what I'm going to do.")
            player.voice.say("So you killed him?")
            self.voice.say("Yeah... You guys thought you were such powerful warriors... such amazing soldiers. Util I ran William through to the hilt and bashed your head in with a mace.")
            player.voice.say("Your a traitor then?")
            self.voice.say("You could call me that. And I accept that title with honor. All my life I've served the king, served you, faught to protect everybody else. This time I want to fight for myself. I want to fight for the things I want. Call me selfish if you like, but I spent a thousand years guarding that gate to protect the mortal realm, now I want to do one thing for myself and you try and kill me. I don't want to fight you Mikkel, I don't want to kill you. But I will")
            player.voice.say("Our oath as a silent warrior was to protect the mortal realm from the dangers of this place. I must complete that mission")
            self.voice.say("Too bad... The heart of magic can only be used for a single wish.")
            player.voice.say("I don't want to fight you Warren. I will part ways with you here.")
            self.voice.say("Be warned Mikkel, I am going to find that heart of magic. Even if I have to kill you to get it.")
        else:
            self.voice.say("Stay away from me Mikkel, or I'll kill you.")
        

            


from code import terminal, map, randomness, util, read
from battle import enemy, battle
from getkey import getkey, keys
import random, time
from classes import weapon, armor
from generators import monsters
from save_game import save

class Game():
    def __init__(self, dungeon, player, characters, posx=False, posy=False):
        """Initiate the Game"""
        self.dungeon = dungeon

        self.player = player

        self.characters = characters

        if not posx and not posy:
            self.y, self.x = map.getSpawn(dungeon)
        else:
            self.y = posy
            self.x = posx


    def next_dungeon(self):
        """Generate a new dungeon and move the player into it"""
        print("The portal swirls before you!")

        print("[1] Enter")
        print("[2] Leave")

        choice = util.get_input("-> ", valid=["1", "2"])

        if choice == '1':
            print("You enter the swirling portal...")
            return True

        else:
            return False

    def play(self):
        """The main Game Loop"""
        count = 0
        
        while True:
            # disabled count clear for now. Manual clear is 9
            count += 1
            if count == 10:
                #terminal.clear() # just to keep things from being messy
                count = 0
                save.save_player_data(self.player, self.dungeon, self.x, self.y)

            stats = [self.player.get_stats()]
            
            map.display(self.dungeon, self.x, self.y, stats)

            self.dungeon[self.y][self.x] = ' '

            key = getkey()

            x = 0
            y = 0
            
            if key == 'w' or key == keys.UP:
                y = -1

            elif key == 's' or key == keys.DOWN:
                y = 1

            elif key == 'd' or key == keys.RIGHT:
                x = +1

            elif key == 'a' or key == keys.LEFT:
                x = -1

            elif key == '1':
                self.player_stats()
            elif key == '2':
                self.player.shuffle_loadout()
            elif key == '3':
                self.player.eat()
            elif key == '4':
                read.pick_book(self.player)
            elif key == '8':
                save.save_player_data(self.player, self.dungeon, self.x, self.y)
            elif key == '9':
                terminal.clear()
            elif key == '0':
                print("Are you Sure you want to exit?")
                print("y/n")
                choice = util.get_input("-> ", valid=['y', 'n'])
                if choice == 'y':
                    print("Saving...")
                    save.save_player_data(self.player, self.dungeon, self.x, self.y)
                    exit()
                elif choice == 'n':
                    terminal.clear()

            action = map.action(self.dungeon, self.x+x, self.y+y)

            if action.valid:
                self.y += y
                self.x += x

            if action.treasure:
                self.open_treasure()
            
            if action.shard:
                self.collect_shard()

            if action.enemy:
                m = monsters.generate_monsters()
                if self.choose_fight(m):
                    self.y += y
                    self.x += x

            if action.portal:
                if self.next_dungeon():
                    return True


            if action.character:
                for character in self.characters:
                    if character.symbol == action.character:
                        character.speak(self.player)

    def player_stats(self):
        """Show the user inventory, armor, etc"""

        terminal.clear()

        width = 70

        util.center(self.player.get_stats(), width)

        print("\n")
        util.center(f"SILVER: {str(self.player.silver)} | GOLD: {str(self.player.gold)} | DIAMONDS: {str(self.player.diamonds)}", width)

        print("\n")
        util.center(f"SHARDS: {str(self.player.shards)}", width)

        print("\n\n")

        util.center("---------- WEAPONS -----------", width)
        print("\n")

        print(self.player.weapon.stats())
        print(self.player.armor.stats())

        print("\n")
        util.center("------------------------------", width)

        print("\n\n")

        util.center("|~ INVENTORY ~|", width)

        print("\n")

        items = []

        for item in sorted(self.player.inventory):
            exist = False
            for i in items:
                if item == i[0]:
                    i[1] += 1
                    exist = True
            if not exist:
                items.append([item, 1])

        for item in items:
            util.indent(f"{item[0]} x{str(item[1])}", 26)
            


        util.cont()



    def open_treasure(self):
        """Gens Random Box and opens it"""

        terminal.clear()

        loot = random.choice(randomness.loot)

        if loot['type'] == 'weapon':
            w = weapon.generate_weapon()
            print('You found a tier ' + str(w.tier) + " " + str(w.name))
            print("[1] Equip")
            print("[2] Keep")
            print("[3] Throw away")
            choice = util.get_input("-> ", valid=["1", "2", "3"])

            if choice == '1':
                self.player.equip_weapon(w)
                self.player.store_weapon(w)

            elif choice == '2':
                self.player.store_weapon(w)

            else:
                print("You threw away the " + w.name)

        elif loot['type'] == 'armor':
            a = armor.generate_armor()
            print('You found tier ' + str(a.tier) + " " + str(a.name))
            print("[1] Equip")
            print("[2] Keep")
            print("[3] Throw away")
            choice = util.get_input("-> ", valid=["1", "2", "3"])

            if choice == '1':
                self.player.equip_armor(a)
                self.player.store_armor(a)

            elif choice == '2':
                self.player.store_armor(a)

            else:
                print("You threw away the " + a.name)

        elif loot['type'] == 'money':
            val = random.randint(1, loot['max'])

            print('You found ' + str(val) + ' ' + loot['name'])

            if loot['name'] == 'gold':
                self.player.gold += val
            elif loot['name'] == 'silver':
                self.player.silver += val
            elif loot['name'] == 'diamond':
                self.player.diamonds += val

        elif loot['type'] == 'inventory':
            print("You found a " + loot['name'])
            self.player.inventory.append(loot['name'])

        util.cont()
            

    def choose_fight(self, enemies):
        if len(enemies) > 1:
            print("A group of " + enemies[0].name + "'s approach!")
        else:
            print("A loan " + enemies[0].name + " approaches")

        print("[1] Battle")
        print("[2] Run")

        choice = util.get_input("-> ", valid=['1', '2'])

        if choice == '1':
            battle.fight(self.player, enemies)
            return True
        else:
            print("You chose to run!")
            
        util.cont()
        
        return False
    
    def collect_shard(self):
        if self.player.shards == 0:
            terminal.slow_print("The small blue crystal glows in your hand")
            terminal.slow_print("It eminates a strange aurora...")
            terminal.slow_print("Something... something familar...")
            self.player.remember_convo1()
        elif self.player.shards == 5:
            terminal.slow_print("Hmm... More of these little things... It's almost as if someone is leaving there here on purpose")
            terminal.slow_print("The more you pick up, the more your start to remember...")
            terminal.slow_print("....................")
            self.player.remember_convo2()
        else:
            print("You found a shard!")
            print("Your magic Powers and Health have grown!")
        
        self.player.shards += 1
        self.player.max_life += 10
        self.player.life += 10
        self.player.max_magic += 5
        self.player.magic += 5
        
        util.cont()

        

        

                
            


from code import terminal, util





def pick_book(player):
	terminal.clear()

	print("Choose Something to Read: ")

	v = ['1', 'exit']

	print("[1] Pack of Letters")

	if "Ferric's Journal" in player.inventory:
		print("[2] Ferric's Journal")
		v.append('2')

	i = util.get_input("-> ", valid=v)

	if i == 'exit':
		return

	if i == '1':
		print("The letters are not marked and old... ")
		print("\n[1] Letter 1")
		print("[2] Letter 2")
		print("[3] Letter 3")
		print("[4] Letter 4")
		print('[5] Back')

		i = util.get_input("-> ", valid=['1', '2', '3', '4', '5'])

		if i == '5':
			pick_book(player)

		else:
			read('story/letters/warren'+i+'.txt')

	elif i == '2':
		read('story/books/ferrics_journal.txt', parts=True)


def read(item, parts=False):
	with open(item, 'r') as file:
		data = file.read()

	if parts:
		content = data.split("PART")
	else:
		content = data.split("\n")


	for i in content:
		terminal.slow_print(i, .001)
		input()

	util.cont()

	terminal.clear()






import random
from code import terminal
from people import chat


class Trekker():
    def __init__(self):
        self.symbol = 'T'
        
        self.voice = chat.Chat("Trekker: ", "\033[34m")

    def speak(self, player):
        terminal.clear()
        if player.trekker_alive:   
            player.trekker_alive = False # RIP

            self.voice.say("M-Mikkel... It's you...")
            player.voice.say("Mikkel? Wh-what happened to you? Your bleeding all over!")
            self.voice.say("Those... those darn ghouls... we- we thought you left us Mikkel")
            player.voice.say("Your dying, I can heal you-")
            self.voice.say("It's too late Mikkel, besides i- i'm tired")
            player.voice.say("I can heal you...")
            self.voice.say("You left us Mikkel! ... you- we were... it's been so many years")
            player.voice.say("I'm sorry-")
            self.voice.say("They died Mikkel! C-coyote, william... ferr-ferric")
            self.voice.say("It's your fault... you could have saved us... we- we should have never... came... here...")

        else:
            print("Trekker is Dead.")
            input()
        

            